@@ -1,12 +1,14 @@
+#include "components/bar.hpp"
+
 #include <algorithm>
 
-#include "components/bar.hpp"
 #include "components/config.hpp"
 #include "components/parser.hpp"
 #include "components/renderer.hpp"
 #include "components/screen.hpp"
 #include "components/taskqueue.hpp"
 #include "components/types.hpp"
+#include "drawtypes/label.hpp"
 #include "events/signal.hpp"
 #include "events/signal_emitter.hpp"
 #include "utils/bspwm.hpp"
@@ -72,46 +74,51 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
   // Get available RandR outputs
   auto monitor_name = m_conf.get(bs, "monitor", ""s);
   auto monitor_name_fallback = m_conf.get(bs, "monitor-fallback", ""s);
-  auto monitor_strictmode = m_conf.get(bs, "monitor-strict", false);
-  auto monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, monitor_strictmode);
+  m_opts.monitor_strict = m_conf.get(bs, "monitor-strict", m_opts.monitor_strict);
+  m_opts.monitor_exact = m_conf.get(bs, "monitor-exact", m_opts.monitor_exact);
+  auto monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, m_opts.monitor_strict, false);
 
   if (monitors.empty()) {
     throw application_error("No monitors found");
   }
 
-  if (monitor_name.empty() && !monitor_strictmode) {
-    auto connected_monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, true);
+  // if monitor_name is not defined, first check for primary monitor
+  if (monitor_name.empty()) {
+    for (auto&& mon : monitors) {
+      if (mon->primary) {
+        monitor_name = mon->name;
+        break;
+      }
+    }
+  }
+
+  // if still not found (and not strict matching), get first connected monitor
+  if (monitor_name.empty() && !m_opts.monitor_strict) {
+    auto connected_monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, true, false);
     if (!connected_monitors.empty()) {
       monitor_name = connected_monitors[0]->name;
       m_log.warn("No monitor specified, using \"%s\"", monitor_name);
     }
   }
 
+  // if still not found, get first monitor
   if (monitor_name.empty()) {
     monitor_name = monitors[0]->name;
     m_log.warn("No monitor specified, using \"%s\"", monitor_name);
   }
 
-  bool name_found{false};
-  bool fallback_found{monitor_name_fallback.empty()};
+  // get the monitor data based on the name
+  m_opts.monitor = randr_util::match_monitor(monitors, monitor_name, m_opts.monitor_exact);
   monitor_t fallback{};
 
-  for (auto&& monitor : monitors) {
-    if (!name_found && (name_found = monitor->match(monitor_name, monitor_strictmode))) {
-      m_opts.monitor = move(monitor);
-    } else if (!fallback_found && (fallback_found = monitor->match(monitor_name_fallback, monitor_strictmode))) {
-      fallback = move(monitor);
-    }
-
-    if (name_found && fallback_found) {
-      break;
-    }
+  if (!monitor_name_fallback.empty()) {
+    fallback = randr_util::match_monitor(monitors, monitor_name_fallback, m_opts.monitor_exact);
   }
 
   if (!m_opts.monitor) {
     if (fallback) {
       m_opts.monitor = move(fallback);
-      m_log.warn("Monitor \"%s\" not found, reverting to fallback \"%s\"", monitor_name, monitor_name_fallback);
+      m_log.warn("Monitor \"%s\" not found, reverting to fallback \"%s\"", monitor_name, m_opts.monitor->name);
     } else {
       throw application_error("Monitor \"" + monitor_name + "\" not found or disconnected");
     }
@@ -150,7 +157,7 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
   // Load configuration values
   m_opts.origin = m_conf.get(bs, "bottom", false) ? edge::BOTTOM : edge::TOP;
   m_opts.spacing = m_conf.get(bs, "spacing", m_opts.spacing);
-  m_opts.separator = m_conf.get(bs, "separator", ""s);
+  m_opts.separator = drawtypes::load_optional_label(m_conf, bs, "separator", "");
   m_opts.locale = m_conf.get(bs, "locale", ""s);
 
   auto radius = m_conf.get<double>(bs, "radius", 0.0);
@@ -190,9 +197,15 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
     }
   }
 
-  const auto parse_or_throw = [&](string key, unsigned int def) -> unsigned int {
+  const auto parse_or_throw_color = [&](string key, rgba def) -> rgba {
     try {
-      return m_conf.get(bs, key, rgba{def});
+      rgba color = m_conf.get(bs, key, def);
+
+      /*
+       * These are the base colors of the bar and cannot be alpha only
+       * In that case, we just use the alpha channel on the default value.
+       */
+      return color.try_apply_alpha_to(def);
     } catch (const exception& err) {
       throw application_error(sstream() << "Failed to set " << key << " (reason: " << err.what() << ")");
     }
@@ -210,37 +223,41 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
       m_log.warn("Ignoring `%s.background` (overridden by gradient background)", bs);
     }
   } else {
-    m_opts.background = parse_or_throw("background", m_opts.background);
+    m_opts.background = parse_or_throw_color("background", m_opts.background);
   }
 
   // Load foreground
-  m_opts.foreground = parse_or_throw("foreground", m_opts.foreground);
+  m_opts.foreground = parse_or_throw_color("foreground", m_opts.foreground);
 
   // Load over-/underline
   auto line_color = m_conf.get(bs, "line-color", rgba{0xFFFF0000});
   auto line_size = m_conf.get(bs, "line-size", 0);
 
   m_opts.overline.size = m_conf.get(bs, "overline-size", line_size);
-  m_opts.overline.color = parse_or_throw("overline-color", line_color);
+  m_opts.overline.color = parse_or_throw_color("overline-color", line_color);
   m_opts.underline.size = m_conf.get(bs, "underline-size", line_size);
-  m_opts.underline.color = parse_or_throw("underline-color", line_color);
+  m_opts.underline.color = parse_or_throw_color("underline-color", line_color);
 
   // Load border settings
   auto border_color = m_conf.get(bs, "border-color", rgba{0x00000000});
-  auto border_size = m_conf.get(bs, "border-size", 0);
+  auto border_size = m_conf.get(bs, "border-size", ""s);
+  auto border_top = m_conf.deprecated(bs, "border-top", "border-top-size", border_size);
+  auto border_bottom = m_conf.deprecated(bs, "border-bottom", "border-bottom-size", border_size);
+  auto border_left = m_conf.deprecated(bs, "border-left", "border-left-size", border_size);
+  auto border_right = m_conf.deprecated(bs, "border-right", "border-right-size", border_size);
 
   m_opts.borders.emplace(edge::TOP, border_settings{});
-  m_opts.borders[edge::TOP].size = m_conf.deprecated(bs, "border-top", "border-top-size", border_size);
-  m_opts.borders[edge::TOP].color = parse_or_throw("border-top-color", border_color);
+  m_opts.borders[edge::TOP].size = geom_format_to_pixels(border_top, m_opts.monitor->h);
+  m_opts.borders[edge::TOP].color = parse_or_throw_color("border-top-color", border_color);
   m_opts.borders.emplace(edge::BOTTOM, border_settings{});
-  m_opts.borders[edge::BOTTOM].size = m_conf.deprecated(bs, "border-bottom", "border-bottom-size", border_size);
-  m_opts.borders[edge::BOTTOM].color = parse_or_throw("border-bottom-color", border_color);
+  m_opts.borders[edge::BOTTOM].size = geom_format_to_pixels(border_bottom, m_opts.monitor->h);
+  m_opts.borders[edge::BOTTOM].color = parse_or_throw_color("border-bottom-color", border_color);
   m_opts.borders.emplace(edge::LEFT, border_settings{});
-  m_opts.borders[edge::LEFT].size = m_conf.deprecated(bs, "border-left", "border-left-size", border_size);
-  m_opts.borders[edge::LEFT].color = parse_or_throw("border-left-color", border_color);
+  m_opts.borders[edge::LEFT].size = geom_format_to_pixels(border_left, m_opts.monitor->w);
+  m_opts.borders[edge::LEFT].color = parse_or_throw_color("border-left-color", border_color);
   m_opts.borders.emplace(edge::RIGHT, border_settings{});
-  m_opts.borders[edge::RIGHT].size = m_conf.deprecated(bs, "border-right", "border-right-size", border_size);
-  m_opts.borders[edge::RIGHT].color = parse_or_throw("border-right-color", border_color);
+  m_opts.borders[edge::RIGHT].size = geom_format_to_pixels(border_right, m_opts.monitor->w);
+  m_opts.borders[edge::RIGHT].color = parse_or_throw_color("border-right-color", border_color);
 
   // Load geometry values
   auto w = m_conf.get(m_conf.section(), "width", "100%"s);
@@ -249,7 +266,7 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
   auto offsety = m_conf.get(m_conf.section(), "offset-y", ""s);
 
   m_opts.size.w = geom_format_to_pixels(w, m_opts.monitor->w);
-  m_opts.size.h = geom_format_to_pixels(h, m_opts.monitor->h);;
+  m_opts.size.h = geom_format_to_pixels(h, m_opts.monitor->h);
   m_opts.offset.x = geom_format_to_pixels(offsetx, m_opts.monitor->w);
   m_opts.offset.y = geom_format_to_pixels(offsety, m_opts.monitor->h);
 
@@ -269,20 +286,9 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
     throw application_error("Resulting bar height is out of bounds (" + to_string(m_opts.size.h) + ")");
   }
 
-  // m_opts.size.w = math_util::cap<int>(m_opts.size.w, 0, m_opts.monitor->w);
-  // m_opts.size.h = math_util::cap<int>(m_opts.size.h, 0, m_opts.monitor->h);
-
-  m_opts.center.y = m_opts.size.h;
-  m_opts.center.y -= m_opts.borders[edge::BOTTOM].size;
-  m_opts.center.y /= 2;
-  m_opts.center.y += m_opts.borders[edge::TOP].size;
-
-  m_opts.center.x = m_opts.size.w;
-  m_opts.center.x -= m_opts.borders[edge::RIGHT].size;
-  m_opts.center.x /= 2;
-  m_opts.center.x += m_opts.borders[edge::LEFT].size;
-
-  m_log.info("Bar geometry: %ix%i+%i+%i", m_opts.size.w, m_opts.size.h, m_opts.pos.x, m_opts.pos.y);
+  m_log.info("Bar geometry: %ix%i+%i+%i; Borders: %d,%d,%d,%d", m_opts.size.w, m_opts.size.h, m_opts.pos.x,
+      m_opts.pos.y, m_opts.borders[edge::TOP].size, m_opts.borders[edge::RIGHT].size, m_opts.borders[edge::BOTTOM].size,
+      m_opts.borders[edge::LEFT].size);
 
   m_log.trace("bar: Attach X event sink");
   m_connection.attach_sink(this, SINK_PRIORITY_BAR);
@@ -320,19 +326,20 @@ void bar::parse(string&& data, bool force) {
 
   std::lock_guard<std::mutex> guard(m_mutex, std::adopt_lock);
 
+  bool unchanged = data == m_lastinput;
+
+  m_lastinput = data;
+
   if (force) {
     m_log.trace("bar: Force update");
   } else if (!m_visible) {
     return m_log.trace("bar: Ignoring update (invisible)");
   } else if (m_opts.shaded) {
     return m_log.trace("bar: Ignoring update (shaded)");
-  } else if (data == m_lastinput) {
+  } else if (unchanged) {
     return m_log.trace("bar: Ignoring update (unchanged)");
-    return;
   }
 
-  m_lastinput = data;
-
   auto rect = m_opts.inner_area();
 
   if (m_tray && !m_tray->settings().detached && m_tray->settings().configured_slots) {
@@ -404,6 +411,11 @@ void bar::show() {
   try {
     m_log.info("Showing bar window");
     m_sig.emit(visibility_change{true});
+    /**
+     * First reconfigures the window so that WMs that discard some information
+     * when unmapping have the correct window properties (geometry etc).
+     */
+    reconfigue_window();
     m_connection.map_window_checked(m_opts.window);
     m_connection.flush();
     m_visible = true;
@@ -460,6 +472,22 @@ void bar::restack_window() {
   }
 }
 
+void bar::reconfigue_window() {
+  m_log.trace("bar: Reconfigure window");
+  restack_window();
+  reconfigure_geom();
+  reconfigure_struts();
+  reconfigure_wm_hints();
+}
+
+/**
+ * Reconfigure window geometry
+ */
+void bar::reconfigure_geom() {
+  window win{m_connection, m_opts.window};
+  win.reconfigure_geom(m_opts.size.w, m_opts.size.h, m_opts.pos.x, m_opts.pos.y);
+}
+
 /**
  * Reconfigure window position
  */
@@ -611,18 +639,22 @@ void bar::handle(const evt::motion_notify& evt) {
   m_log.trace("bar: Detected motion: %i at pos(%i, %i)", evt->detail, evt->event_x, evt->event_y);
 #if WITH_XCURSOR
   m_motion_pos = evt->event_x;
-  // scroll cursor is less important than click cursor, so we shouldn't return until we are sure there is no click action
+  // scroll cursor is less important than click cursor, so we shouldn't return until we are sure there is no click
+  // action
   bool found_scroll = false;
   const auto find_click_area = [&](const action& action) {
-    if (!m_opts.cursor_click.empty() && !(action.button == mousebtn::SCROLL_UP || action.button == mousebtn::SCROLL_DOWN || action.button == mousebtn::NONE)) {
+    if (!m_opts.cursor_click.empty() &&
+        !(action.button == mousebtn::SCROLL_UP || action.button == mousebtn::SCROLL_DOWN ||
+            action.button == mousebtn::NONE)) {
       if (!string_util::compare(m_opts.cursor, m_opts.cursor_click)) {
         m_opts.cursor = m_opts.cursor_click;
         m_sig.emit(cursor_change{string{m_opts.cursor}});
       }
       return true;
-    } else if (!m_opts.cursor_scroll.empty() && (action.button == mousebtn::SCROLL_UP || action.button == mousebtn::SCROLL_DOWN)) {
+    } else if (!m_opts.cursor_scroll.empty() &&
+               (action.button == mousebtn::SCROLL_UP || action.button == mousebtn::SCROLL_DOWN)) {
       if (!found_scroll) {
-          found_scroll = true;
+        found_scroll = true;
       }
     }
     return false;
@@ -631,11 +663,11 @@ void bar::handle(const evt::motion_notify& evt) {
   for (auto&& action : m_renderer->actions()) {
     if (action.test(m_motion_pos)) {
       m_log.trace("Found matching input area");
-      if(find_click_area(action))
+      if (find_click_area(action))
         return;
     }
   }
-  if(found_scroll) {
+  if (found_scroll) {
     if (!string_util::compare(m_opts.cursor, m_opts.cursor_scroll)) {
       m_opts.cursor = m_opts.cursor_scroll;
       m_sig.emit(cursor_change{string{m_opts.cursor}});
@@ -645,11 +677,11 @@ void bar::handle(const evt::motion_notify& evt) {
   for (auto&& action : m_opts.actions) {
     if (!action.command.empty()) {
       m_log.trace("Found matching fallback handler");
-      if(find_click_area(action))
+      if (find_click_area(action))
         return;
     }
   }
-  if(found_scroll) {
+  if (found_scroll) {
     if (!string_util::compare(m_opts.cursor, m_opts.cursor_scroll)) {
       m_opts.cursor = m_opts.cursor_scroll;
       m_sig.emit(cursor_change{string{m_opts.cursor}});
@@ -790,17 +822,13 @@ bool bar::on(const signals::eventqueue::start&) {
   if (m_opts.dimvalue != 1.0) {
     m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_ENTER_WINDOW | XCB_EVENT_MASK_LEAVE_WINDOW);
   }
-  if (!m_opts.cursor_click.empty() || !m_opts.cursor_scroll.empty() ) {
+  if (!m_opts.cursor_click.empty() || !m_opts.cursor_scroll.empty()) {
     m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_POINTER_MOTION);
   }
   m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_STRUCTURE_NOTIFY);
 
   m_log.info("Bar window: %s", m_connection.id(m_opts.window));
-  restack_window();
-
-  m_log.trace("bar: Reconfigure window");
-  reconfigure_struts();
-  reconfigure_wm_hints();
+  reconfigue_window();
 
   m_log.trace("bar: Map window");
   m_connection.map_window_checked(m_opts.window);
@@ -810,7 +838,6 @@ bool bar::on(const signals::eventqueue::start&) {
   m_sig.emit(signals::ui::update_geometry{});
 
   // Reconfigure window position after mapping (required by Openbox)
-  // Required by Openbox
   reconfigure_pos();
 
   m_log.trace("bar: Draw empty bar");
@@ -839,7 +866,8 @@ bool bar::on(const signals::ui::unshade_window&) {
   double steptime{25.0 / 2.0};
   m_anim_step = distance / steptime / 2.0;
 
-  m_taskqueue->defer_unique("window-shade", 25ms,
+  m_taskqueue->defer_unique(
+      "window-shade", 25ms,
       [&](size_t remaining) {
         if (!m_opts.shaded) {
           m_sig.emit(signals::ui::tick{});
@@ -878,7 +906,8 @@ bool bar::on(const signals::ui::shade_window&) {
   double steptime{25.0 / 2.0};
   m_anim_step = distance / steptime / 2.0;
 
-  m_taskqueue->defer_unique("window-shade", 25ms,
+  m_taskqueue->defer_unique(
+      "window-shade", 25ms,
       [&](size_t remaining) {
         if (m_opts.shaded) {
           m_sig.emit(signals::ui::tick{});
@@ -938,7 +967,7 @@ bool bar::on(const signals::ui::dim_window& sig) {
 
 #if WITH_XCURSOR
 bool bar::on(const signals::ui::cursor_change& sig) {
-  if(!cursor_util::set_cursor(m_connection, m_connection.screen(), m_opts.window, sig.cast())) {
+  if (!cursor_util::set_cursor(m_connection, m_connection.screen(), m_opts.window, sig.cast())) {
     m_log.warn("Failed to create cursor context");
   }
   m_connection.flush();
